#include "mcc_generated_files/mcc.h"
#include "mcc_generated_files/heartrate.h"
#include "mcc_generated_files/eusart2.h"
#include "SpO2_GPS_SMS.h"
//***************************************************
//Local Defines and Variales
//***************************************************
#define responseBufferSize  200    //Size of Buffer to store GSM-GPS Module's Response 
uint16_t get_ir=0;                 //Variable to store InfraRedLed Value of HR Click
uint16_t get_red=0;                //Variable to store RedLed Value of HR Click     
double AVG_IR=0;                   //Variable to store AVERAGE of InfraRedLed Samples
double AVG_RED=0;                  //Variable to store AVERAGE of RedLed Samples
unsigned int NumberOfRedIrSamples=0; //Variable to store Number of InfraRedLed  and RedLed Samples required to Calculate SpO2

uint8_t SpO2_initialized = 0;      //SpO2 initialization Flag
float SpO2;                        //Variable to Store SpO2 value 
char* gpscurrentLocation;          //Char Pointer to store Current location
static char gsmResponseBuffer [responseBufferSize] = {0};   //Buffer to SOTRE GSM-GPS Module's Response 
char* exampleRead;                 // Char Pointer to READ GSM-GPS Module's Response
static uint8_t gsmResponseIndex = 0;//Variable to store the number of characters received from GSM-GPS Module as Response 

void GSM_COMMANDS(char *,unsigned int);

//**********************************************************
//Initialize SpO2/HR click
//**********************************************************
void SpO2_Initialize()
{
    SpO2_readSensors();
} 

//**********************************************
//Initialize GSM_GPS module
//*********************************************
void GSM_GPS_Initialize() 
 {
    GSM_GPS_Hardware_Reset();          //Toggle for HW Reset
    __delay_ms(40000);                 //Wait for Module Information Response  
    GSM_GPS_Read_Write_Ready_Buffer(); //Read and Print GSM-GPS Module Response  
    for(int Fix_Baud=0;Fix_Baud<10;Fix_Baud++)
    {
      GSM_COMMANDS("AT\r",1000);        //Send "AT" for Baud Rate Negotiation                                                      
    }      
    GSM_COMMANDS("AT\r",1000);          //Send "ATE0" to disable command ECHOs               
    GSM_COMMANDS("AT+CGPSPWR=1\r",2000);//Power On GPS               
    GSM_COMMANDS("AT+IPR=9600\r",1000);//Set Baud Rate to 9600                  
    GSM_COMMANDS("AT&W\r",1000);       //Save the parameter settings to retain on Restart-BaudRate & ECHO Mode OFF          
    GSM_COMMANDS("AT+CMGF=1\r",1000);  //Set the SMS Text MODE Not PDU Mode
    GSM_COMMANDS("AAT+CGPSSTATUS?\r",5000); //Check whether the GPS location is Fixed    
  }
 
//****************************************************************************
//Read InfraRed LED and RED LED samples,Average the Samples and Calculate SpO2
//*****************************************************************************
 void COMPUTE_SPO2()
 {
     PRINT_DATA=USART2_TX;                   //Enable Flag to Send data from PIC to PC
     printf("#################################Place your finger########################## \n");       
     while(NumberOfRedIrSamples<100)           //Check for Number Of Samples  
         {
        SpO2_readSensors();                  //Initialize HR click in Pulse Oxy-meter Mode 
        get_ir=SpO2_getIRdata();             //Read  RED Values     
        get_red=SpO2_getREDdata();           //Read IR Values
            if((get_ir>1000) && (get_red>1000))     //if hand is placed,start to sample.
              {
              AVG_IR=get_ir+AVG_IR;          //Sum of RedLed Samples
              AVG_RED=get_red+AVG_RED;       //Sum of InfraRedLed Samples.
             NumberOfRedIrSamples++;         //Decrement for each sample until Zero
              }   
              else
              {
             SpO2_Sample_Init();             //if hand is off restart the sample
             }       
          }
         AVG_IR=AVG_IR/NumberOfRedIrSamples;  //Average InfraRedLed samples
         AVG_RED=AVG_RED/NumberOfRedIrSamples;//Average RedLed samples
         AVG_IR=(AVG_IR)*950;                 //(Average Of InraRedLed Values)/(wavelength of InfraRedLed)
         AVG_RED=(AVG_RED)*650;               //(Average Of RedLed Values)/(Wavelength of RedLed)
         SpO2=((110-25*(AVG_RED/AVG_IR)));    //Formula to Calculate SpO2
         SpO2_Sample_Init();                  //Re-initialize for next reading 
 }   
      
 //***********************************************
 //Reset SpO2 sample variables 
 //***********************************************
 void SpO2_Sample_Init()
 {
     NumberOfRedIrSamples=0;
     AVG_IR=0;
     AVG_RED=0;     
 }
 
 //********************************************************************
 //     Initialize the PulseOxyMeter
 //********************************************************************
void SpO2_initializeClick(void) {
    MAX30100_setSpo2RdyInterrupt(SPO2_INTERRUPT_EN);  //Configure SP02 Interrupt
    MAX30100_setHrRdyInterrupt(HR_INTERRUPT_EN);      //Configure HR Ready Interrupt
    MAX30100_setTempRdyInterrupt(TEMP_INTERRUPT_EN);  //Configure Temperature Ready Interrupt
    MAX30100_setFifoAfullInterrupt(FIFO_INTERRUPT_EN);//FIFO Full Interrupt Enable

    MAX30100_setMode(DEFAULT_MODE);                   //Set HeartRate/PulseOxyMeter Mode
    MAX30100_setHiResEnabled(DEFAULT_HIRES_SET);   
    MAX30100_setSampleRate(DEFAULT_SAMP_RATE);       //Set The Sampling Rate of ADC
    MAX30100_setPulseWidth(DEFAULT_PWIDTH);          //Set Pulse Width of the LEDs.
    MAX30100_setIRLEDCurrent(DEFAULT_IR_CURRENT);    //Set InfraRedLed Current
    MAX30100_setREDLEDCurrent(DEFAULT_RED_CURRENT);  //Set REDLed Current

    MAX30100_initializeSensor();                     // Initialize HR/SpO2 click
    SpO2_initialized = 1;                            //Flag for SpO2 Initialization
}
//********************************************************************
//      Read and Return the InfraRedLed value of PulseOxyMeter
//********************************************************************
uint16_t SpO2_getIRdata(void) {
    return MAX30100_getIRdata();
}

//********************************************************************
//      Read and Return the RedLed Value of PulseOxyMeter
//********************************************************************
uint16_t SpO2_getREDdata(void) {
    return MAX30100_getREDdata();    
}


 
 //******************************************************************
 //                 Read GPS Location
 //******************************************************************
 void Read_GPS_LOCATION()
 {
    PRINT_DATA=USART1_TX;              //Enable Flag to Transmit data from PIC to GSM/GPS
    printf("AT+CGPSINF=32\r");         //Get Current GPS Info
    __delay_ms(5000);                  //Wait for Module Information Response  
    exampleRead=Read_GSM_GPS_Buffer(); //Read Response           
    gpscurrentLocation=exampleRead;    //Save current location to send SMS      
 }
  
 
 //********************************************************************
 //             Send LOCATION and SpO2 as SMS to a Phone Number
 //********************************************************************
 void TextMessage_GSM_GPS_SPO2()      
       {   
         /**Send Location and SpO2 to PC for reference **/
          PRINT_DATA=USART2_TX;                              //Enable Flag to Transmit data from PIC to PC
          printf("Location  %s \n",gpscurrentLocation);     //GPS Current Location       
          printf("Your SPO2 %f %c  \n\n\n\n",SpO2,'%');     //SpO2
             
          /**Send Location and SpO2 to a Phone Number**/  
          PRINT_DATA=USART1_TX ;                            //Enable Flag to Transmit data from PIC to GSM/GPS
         // printf("AT+CMGS=\"+1xxxxxxxxxx\"\r");           //Number to Receive SMS(Enter a Number)  
          __delay_ms(1000);  
          printf("LOCATION: %s \n",gpscurrentLocation);//GPS Current Location 
          printf("Your SPO2: %f %c",SpO2,'%');                //Value of SpO2 
          __delay_ms(1000);                                //Wait to complete transmission
          printf("\x1A");                                  //Ctl-Z ,End Of SMS      
          __delay_ms(1000);                                //Wait for GSM-GPS Module Response
          
          Ready_GSM_GPS_Buffer();                          //Prepare for Next message   
 }
 
//**********************************************
// Toggle GSM-GPS Module HW Reset
//**********************************************
 void GSM_GPS_Hardware_Reset()        
{    
    PWKEY_SetLow();
    __delay_ms(2000); 
     PWKEY_SetHigh();   
}
 
 void GSM_COMMANDS(char *command,unsigned int delay)
 {
     PRINT_DATA=USART1_TX;            //Enable Flag to Transmit data from PIC to GSM/GPS
     printf("%s",command);           //Command to GSM_GPS module
     __delay_ms(delay);             //Wait for Module Information Response  
     GSM_GPS_Read_Write_Ready_Buffer();  //Read and print GSM-GPS Module Response 
 }
 
 //********************************************************************
 //           Read the Data Received form GSM_GPS Module
 //********************************************************************
 static char* Read_GSM_GPS_Buffer()
 {
    while( eusart2RxCount!=0)
    {
     gsmResponseBuffer[gsmResponseIndex++]=EUSART2_Read();
    }   
    return gsmResponseBuffer;
 }
 
//********************************************************************
//      Send the data Received form GSM_GPS Module to PC
//******************************************************************** 
 void Write_GSM_GPS_Read()      //RB7-USART2
{   
       PRINT_DATA=USART2_TX;    //Enable Flag to Transmit data from PIC to PC      
       printf("%s",exampleRead); //Print the data Received from GSM-GPS Module. 
} 
 
//********************************************************************
//     Ready the Buffer to Receive Data form GSM_GPS Module 
//********************************************************************
  static void Ready_GSM_GPS_Buffer()
 {
    for(uint8_t position=0;position< responseBufferSize;position++)
         gsmResponseBuffer[position]=0;
    gsmResponseIndex=0;
 }
 
//********************************************************************
// Read and Print the Response from GSM-GPS module 
// and Ready the Buffer to store the next response      
//********************************************************************
  void GSM_GPS_Read_Write_Ready_Buffer()
  {
  exampleRead=Read_GSM_GPS_Buffer(); //Read Response
  Write_GSM_GPS_Read();              //Print Response for reference     
  Ready_GSM_GPS_Buffer();            //Prepare for Next message   
  }
 
//********************************************************************
//      Read the IR and RED values of the PulseOxyMeter
//********************************************************************
   void SpO2_readSensors(void) {
    if (!SpO2_initialized) {          //Check for Initialization
        SpO2_initializeClick();       //Initialize HR Click
    }
    MAX30100_readSensors();           //Read InfraRedLed and RedLed values from FIFO
}

 
 
 
